@@ -78,6 +78,8 @@ if TYPE_CHECKING:
     from typing import TypeAlias
 
     from hypothesis.strategies import SearchStrategy
+    from hypothesis.strategies._internal.core import DataObject
+    from hypothesis.strategies._internal.random import RandomState
     from hypothesis.strategies._internal.strategies import Ex
 
 
@@ -276,16 +278,16 @@ class SpanProperty:
         """Rerun the test case with this visitor and return the
         results of ``self.finish()``."""
         for record in self.spans.trail:
-            if record == TrailType.CHOICE:
+            if record == TrailType.STOP_SPAN_DISCARD:
+                self.__pop(discarded=True)
+            elif record == TrailType.STOP_SPAN_NO_DISCARD:
+                self.__pop(discarded=False)
+            elif record == TrailType.CHOICE:
                 self.choice_count += 1
-            elif record >= TrailType.START_SPAN:
-                self.__push(record - TrailType.START_SPAN)
             else:
-                assert record in (
-                    TrailType.STOP_SPAN_DISCARD,
-                    TrailType.STOP_SPAN_NO_DISCARD,
-                )
-                self.__pop(discarded=record == TrailType.STOP_SPAN_DISCARD)
+                # everything after TrailType.CHOICE is the label of a span start.
+                self.__push(record - TrailType.CHOICE - 1)
+
         return self.finish()
 
     def __push(self, label_index: int) -> None:
@@ -316,8 +318,10 @@ class SpanProperty:
 class TrailType(IntEnum):
     STOP_SPAN_DISCARD = 1
     STOP_SPAN_NO_DISCARD = 2
-    START_SPAN = 3
-    CHOICE = calc_label_from_name("ir draw record")
+    CHOICE = 3
+    # every trail element larger than TrailType.CHOICE is the label of a span
+    # start, offset by its index. So the first span label is stored as 4, the
+    # second as 5, etc, regardless of its actual integer label.
 
 
 class SpanRecord:
@@ -350,7 +354,7 @@ class SpanRecord:
         except KeyError:
             i = self.__index_of_labels.setdefault(label, len(self.labels))
             self.labels.append(label)
-        self.trail.append(TrailType.START_SPAN + i)
+        self.trail.append(TrailType.CHOICE + 1 + i)
 
     def stop_span(self, *, discard: bool) -> None:
         if discard:
@@ -659,7 +663,6 @@ class ConjectureData:
         self.observer = observer
         self.max_choices = max_choices
         self.max_length = BUFFER_SIZE
-        self.is_find = False
         self.overdraw = 0
         self._random = random
 
@@ -713,10 +716,15 @@ class ConjectureData:
         self._observability_predicates: defaultdict[str, PredicateCounts] = defaultdict(
             PredicateCounts
         )
+
         self._sampled_from_all_strategies_elements_message: Optional[
             tuple[str, object]
         ] = None
         self._shared_strategy_draws: dict[Hashable, tuple[int, Any]] = {}
+        self._shared_data_strategy: Optional[DataObject] = None
+        self._stateful_repr_parts: Optional[list[Any]] = None
+        self.states_for_ids: Optional[dict[int, RandomState]] = None
+        self.seeds_to_states: Optional[dict[Any, RandomState]] = None
         self.hypothesis_runner = not_set
 
         self.expected_exception: Optional[BaseException] = None
@@ -1189,16 +1197,10 @@ class ConjectureData:
         label: Optional[int] = None,
         observe_as: Optional[str] = None,
     ) -> "Ex":
-        from hypothesis.internal.observability import TESTCASE_CALLBACKS
+        from hypothesis.internal.observability import observability_enabled
         from hypothesis.strategies._internal.lazy import unwrap_strategies
         from hypothesis.strategies._internal.utils import to_jsonable
 
-        if self.is_find and not strategy.supports_find:
-            raise InvalidArgument(
-                f"Cannot use strategy {strategy!r} within a call to find "
-                "(presumably because it would be invalid after the call had ended)."
-            )
-
         at_top_level = self.depth == 0
         start_time = None
         if at_top_level:
@@ -1244,7 +1246,7 @@ class ConjectureData:
                     f"while generating {key.removeprefix('generate:')!r} from {strategy!r}",
                 )
                 raise
-            if TESTCASE_CALLBACKS:
+            if observability_enabled():
                 avoid = self.provider.avoid_realization
                 self._observability_args[key] = to_jsonable(v, avoid_realization=avoid)
             return v
@@ -1360,9 +1362,7 @@ class ConjectureData:
         self.freeze()
         raise StopTest(self.testcounter)
 
-    def mark_interesting(
-        self, interesting_origin: Optional[InterestingOrigin] = None
-    ) -> NoReturn:
+    def mark_interesting(self, interesting_origin: InterestingOrigin) -> NoReturn:
         self.conclude_test(Status.INTERESTING, interesting_origin)
 
     def mark_invalid(self, why: Optional[str] = None) -> NoReturn:
